@@ -53,7 +53,7 @@ enum
 {
    /** No redundancy, single NIC mode */
    ECT_RED_NONE,
-   /** Double redundant NIC connecetion */
+   /** Double redundant NIC connection */
    ECT_RED_DOUBLE
 };
 
@@ -151,10 +151,11 @@ int ecx_setupnic(ecx_portt *port, const char *ifname, int secondary)
      sprintf(fname, "/dev/bpf%i", i);
      *bpf = open(fname, O_RDWR);
    }
-   
+
    if(*bpf == -1)
    {
      /* Failed to open bpf */
+     perror("Could not open bpf\n");
      return 0;
    }
 
@@ -164,40 +165,52 @@ int ecx_setupnic(ecx_portt *port, const char *ifname, int secondary)
    if (ioctl(*bpf, BIOCSBLEN, &buffer_len) == -1) {
      perror("BIOCGBLEN");
    }
-   assert(buffer_len >= 1518);
+   
+   if (buffer_len < 1518) {
+     printf("buffer_len %d < 1518\n", buffer_len);
+     return 0;
+   }
 
    /* connect bpf to NIC by name */
    strcpy(ifr.ifr_name, ifname);
-   assert(ioctl(*bpf, BIOCSETIF, &ifr) == 0);
+   if(ioctl(*bpf, BIOCSETIF, &ifr) == -1) {
+     perror("BIOCSETIF");
+     return 0;
+   }
 
    /* Set required bpf options */
 
    /* Activate immediate mode */
    if (ioctl(*bpf, BIOCIMMEDIATE, &true_val) == -1) {
      perror("BIOCIMMEDIATE");
+     return 0;
    }
 
    /* Set interface in promiscuous mode */
    if (ioctl(*bpf, BIOCPROMISC, &true_val) == -1) {
      perror("BIOCPROMISC");
+     return 0;
    }
    
    /* Allow to have custom source address */
    if (ioctl(*bpf, BIOCSHDRCMPLT, &true_val) == -1) {
      perror("BIOCSHDRCMPLT");
+     return 0;
    }
 
    /* Listen only to incomming messages */
    uint direction = BPF_D_IN;
    if (ioctl(*bpf, BIOCSDIRECTION, &direction) == -1) {
      perror("BIOCSDIRECTION");
+     return 0;
    }
 
    /* Set read timeout */
    timeout.tv_sec =  0;
-   timeout.tv_usec = 1;
+   timeout.tv_usec = 11000;
    if (ioctl(*bpf, BIOCSRTIMEOUT, &timeout) == -1) {
      perror("BIOCSRTIMEOUT");
+     return 0;
    }
 
    /* setup ethernet headers in tx buffers so we don't have to repeat it */
@@ -207,9 +220,8 @@ int ecx_setupnic(ecx_portt *port, const char *ifname, int secondary)
       port->rxbufstat[i] = EC_BUF_EMPTY;
    }
    ec_setupheader(&(port->txbuf2));
-   if (r == 0) rval = 1;
 
-   return rval;
+   return 1;
 }
 
 /** Close sockets used
@@ -315,8 +327,12 @@ int ecx_outframe(ecx_portt *port, int idx, int stacknumber)
    }
    lp = (*stack->txbuflength)[idx];
    //rval = send(*stack->sock, (*stack->txbuf)[idx], lp, 0);
-   rval = write (*stack->sock,(*stack->txbuf)[idx], lp);
    (*stack->rxbufstat)[idx] = EC_BUF_TX;
+   rval = write (*stack->sock,(*stack->txbuf)[idx], lp);
+   if (rval == -1)
+   {
+      (*stack->rxbufstat)[idx] = EC_BUF_EMPTY;
+   }
 
    return rval;
 }
@@ -349,9 +365,12 @@ int ecx_outframe_red(ecx_portt *port, int idx)
       ehp->sa1 = htons(secMAC[1]);
       /* transmit over secondary socket */
       //send(port->redport->sockhandle, &(port->txbuf2), port->txbuflength2 , 0);
-      write(port->redport->sockhandle, &(port->txbuf2), port->txbuflength2);
-      pthread_mutex_unlock( &(port->tx_mutex) );
       port->redport->rxbufstat[idx] = EC_BUF_TX;
+      if (write(port->redport->sockhandle, &(port->txbuf2), port->txbuflength2) == -1)
+      {
+         (*stack->rxbufstat)[idx] = EC_BUF_EMPTY;
+      }
+      pthread_mutex_unlock( &(port->tx_mutex) );
    }
 
    return rval;
@@ -386,7 +405,7 @@ static int ecx_recvpkt(ecx_portt *port, int stacknumber)
 /** Non blocking receive frame function. Uses RX buffer and index to combine
  * read frame with transmitted frame. To compensate for received frames that
  * are out-of-order all frames are stored in their respective indexed buffer.
- * If a frame was placed in the buffer previously, the function retreives it
+ * If a frame was placed in the buffer previously, the function retrieves it
  * from that buffer index without calling ec_recvpkt. If the requested index
  * is not already in the buffer it calls ec_recvpkt to fetch it. There are
  * three options now, 1 no frame read, so exit. 2 frame read but other
@@ -445,7 +464,7 @@ int ecx_inframe(ecx_portt *port, int idx, int stacknumber)
             ecp =(ec_comt*)(&ehp[1]);
             l = etohs(ecp->elength) & 0x0fff;
             idxf = ecp->index;
-            /* found index equals reqested index ? */
+            /* found index equals requested index ? */
             if (idxf == idx)
             {
                /* yes, put it in the buffer array (strip headers) */
@@ -471,7 +490,7 @@ int ecx_inframe(ecx_portt *port, int idx, int stacknumber)
                }
                else
                {
-                  /* strange things happend */
+                  /* strange things happened */
                }
             }
          }
@@ -480,7 +499,7 @@ int ecx_inframe(ecx_portt *port, int idx, int stacknumber)
 
    }
 
-   /* WKC if mathing frame found */
+   /* WKC if matching frame found */
    return rval;
 }
 
@@ -523,10 +542,10 @@ static int ecx_waitinframe_red(ecx_portt *port, int idx, osal_timert *timer)
    /* only do redundant functions when in redundant mode */
    if (port->redstate != ECT_RED_NONE)
    {
-      /* primrx if the reveived MAC source on primary socket */
+      /* primrx if the received MAC source on primary socket */
       primrx = 0;
       if (wkc > EC_NOFRAME) primrx = port->rxsa[idx];
-      /* secrx if the reveived MAC source on psecondary socket */
+      /* secrx if the received MAC source on psecondary socket */
       secrx = 0;
       if (wkc2 > EC_NOFRAME) secrx = port->redport->rxsa[idx];
 
@@ -586,16 +605,11 @@ int ecx_waitinframe(ecx_portt *port, int idx, int timeout)
 
    osal_timer_start (&timer, timeout);
    wkc = ecx_waitinframe_red(port, idx, &timer);
-   /* if nothing received, clear buffer index status so it can be used again */
-   if (wkc <= EC_NOFRAME)
-   {
-      ecx_setbufstat(port, idx, EC_BUF_EMPTY);
-   }
 
    return wkc;
 }
 
-/** Blocking send and recieve frame function. Used for non processdata frames.
+/** Blocking send and receive frame function. Used for non processdata frames.
  * A datagram is build into a frame and transmitted via this function. It waits
  * for an answer and returns the workcounter. The function retries if time is
  * left and the result is WKC=0 or no frame received.
@@ -623,18 +637,13 @@ int ecx_srconfirm(ecx_portt *port, int idx, int timeout)
       }
       else
       {
-         /* normally use partial timout for rx */
+         /* normally use partial timeout for rx */
          osal_timer_start (&timer2, EC_TIMEOUTRET);
       }
       /* get frame from primary or if in redundant mode possibly from secondary */
       wkc = ecx_waitinframe_red(port, idx, &timer2);
    /* wait for answer with WKC>=0 or otherwise retry until timeout */
    } while ((wkc <= EC_NOFRAME) && !osal_timer_is_expired (&timer1));
-   /* if nothing received, clear buffer index status so it can be used again */
-   if (wkc <= EC_NOFRAME)
-   {
-      ecx_setbufstat(port, idx, EC_BUF_EMPTY);
-   }
 
    return wkc;
 }
