@@ -19,9 +19,7 @@
 #include <string.h>
 #include "osal.h"
 #include "oshw.h"
-#include "ethercattype.h"
-#include "ethercatbase.h"
-#include "ethercatmain.h"
+#include "ethercat.h"
 
 
 /** delay in us for eeprom ready loop */
@@ -112,15 +110,15 @@ ecx_contextt  ecx_context = {
     &ec_elist,          // .elist         =
     &ec_idxstack,       // .idxstack      =
     &EcatError,         // .ecaterror     =
-    0,                  // .DCtO          =
-    0,                  // .DCl           =
     &ec_DCtime,         // .DCtime        =
     &ec_SMcommtype[0],  // .SMcommtype    =
     &ec_PDOassign[0],   // .PDOassign     =
     &ec_PDOdesc[0],     // .PDOdesc       =
     &ec_SM,             // .eepSM         =
     &ec_FMMU,           // .eepFMMU       =
-    NULL                // .FOEhook()
+    NULL,               // .FOEhook()
+    NULL,               // .EOEhook()
+    0                   // .manualstatechange
 };
 #endif
 
@@ -325,7 +323,7 @@ void ecx_close(ecx_contextt *context)
 
 /** Read one byte from slave EEPROM via cache.
  *  If the cache location is empty then a read request is made to the slave.
- *  Depending on the slave capabillities the request is 4 or 8 bytes.
+ *  Depending on the slave capabilities the request is 4 or 8 bytes.
  *  @param[in] context = context struct
  *  @param[in] slave   = slave number
  *  @param[in] address = eeprom address in bytes (slave uses words)
@@ -334,7 +332,8 @@ void ecx_close(ecx_contextt *context)
 uint8 ecx_siigetbyte(ecx_contextt *context, uint16 slave, uint16 address)
 {
    uint16 configadr, eadr;
-   uint64 edat;
+   uint64 edat64;
+   uint32 edat32;
    uint16 mapw, mapb;
    int lp,cnt;
    uint8 retval;
@@ -360,17 +359,18 @@ uint8 ecx_siigetbyte(ecx_contextt *context, uint16 slave, uint16 address)
          configadr = context->slavelist[slave].configadr;
          ecx_eeprom2master(context, slave); /* set eeprom control to master */
          eadr = address >> 1;
-         edat = ecx_readeepromFP (context, configadr, eadr, EC_TIMEOUTEEP);
+         edat64 = ecx_readeepromFP (context, configadr, eadr, EC_TIMEOUTEEP);
          /* 8 byte response */
          if (context->slavelist[slave].eep_8byte)
          {
-            put_unaligned64(edat, &(context->esibuf[eadr << 1]));
+            put_unaligned64(edat64, &(context->esibuf[eadr << 1]));
             cnt = 8;
          }
          /* 4 byte response */
          else
          {
-            put_unaligned32(edat, &(context->esibuf[eadr << 1]));
+            edat32 = (uint32)edat64;
+            put_unaligned32(edat32, &(context->esibuf[eadr << 1]));
             cnt = 4;
          }
          /* find bitmap location */
@@ -724,11 +724,19 @@ int ecx_readstate(ecx_contextt *context)
    ec_alstatust sl[MAX_FPRD_MULTI];
    uint16 slca[MAX_FPRD_MULTI];
    boolean noerrorflag, allslavessamestate;
+   boolean allslavespresent = FALSE;
+   int wkc;
 
-   /* Try to establish the state of all slaves sending only one broadcast datargam.
+   /* Try to establish the state of all slaves sending only one broadcast datagram.
     * This way a number of datagrams equal to the number of slaves will be sent only if needed.*/
    rval = 0;
-   ecx_BRD(context->port, 0, ECT_REG_ALSTAT, sizeof(rval), &rval, EC_TIMEOUTRET);
+   wkc = ecx_BRD(context->port, 0, ECT_REG_ALSTAT, sizeof(rval), &rval, EC_TIMEOUTRET);
+
+   if(wkc >= *(context->slavecount))
+   {
+      allslavespresent = TRUE;
+   }
+
    rval = etohs(rval);
    bitwisestate = (rval & 0x0f);
 
@@ -757,7 +765,7 @@ int ecx_readstate(ecx_contextt *context)
          break;
    }
     
-   if (noerrorflag && allslavessamestate)
+   if (noerrorflag && allslavessamestate && allslavespresent)
    {
       /* No slave has toggled the error flag so the alstatuscode
        * (even if different from 0) should be ignored and
@@ -846,7 +854,7 @@ int ecx_writestate(ecx_contextt *context, uint16 slave)
  * @param[in] context     = context struct
  * @param[in] slave       = Slave number, 0 = all slaves (only the "slavelist[0].state" is refreshed)
  * @param[in] reqstate    = Requested state
- * @param[in] timeout     = Timout value in us
+ * @param[in] timeout     = Timeout value in us
  * @return Requested state, or found state after timeout.
  */
 uint16 ecx_statecheck(ecx_contextt *context, uint16 slave, uint16 reqstate, int timeout)
@@ -1032,7 +1040,7 @@ int ecx_mbxreceive(ecx_contextt *context, uint16 slave, ec_mbxbuft *mbx, int tim
                ecx_mbxerror(context, slave, etohs(MBXEp->Detail));
                wkc = 0; /* prevent emergency to cascade up, it is already handled. */
             }
-            else if ((wkc > 0) && ((mbxh->mbxtype & 0x0f) == 0x03)) /* CoE response? */
+            else if ((wkc > 0) && ((mbxh->mbxtype & 0x0f) == ECT_MBXT_COE)) /* CoE response? */
             {
                EMp = (ec_emcyt *)mbx;
                if ((etohs(EMp->CANOpen) >> 12) == 0x01) /* Emergency request? */
@@ -1042,6 +1050,25 @@ int ecx_mbxreceive(ecx_contextt *context, uint16 slave, ec_mbxbuft *mbx, int tim
                   wkc = 0; /* prevent emergency to cascade up, it is already handled. */
                }
             }
+            else if ((wkc > 0) && ((mbxh->mbxtype & 0x0f) == ECT_MBXT_EOE)) /* EoE response? */
+            {
+               ec_EOEt * eoembx = (ec_EOEt *)mbx;
+               uint16 frameinfo1 = etohs(eoembx->frameinfo1);
+               /* All non fragment data frame types are expected to be handled by
+               * slave send/receive API if the EoE hook is set
+               */
+               if (EOE_HDR_FRAME_TYPE_GET(frameinfo1) == EOE_FRAG_DATA)
+               {
+                  if (context->EOEhook)
+                  {
+                     if (context->EOEhook(context, slave, eoembx) > 0)
+                     {
+                        /* Fragment handled by EoE hook */
+                        wkc = 0;
+                     }
+                  }
+               }
+            }
             else
             {
                if (wkc <= 0) /* read mailbox lost */
@@ -1069,7 +1096,8 @@ int ecx_mbxreceive(ecx_contextt *context, uint16 slave, ec_mbxbuft *mbx, int tim
       }
       else /* no read mailbox available */
       {
-          wkc = 0;
+         if (wkc > 0)
+            wkc = EC_TIMEOUT;
       }
    }
 
@@ -1611,14 +1639,16 @@ uint32 ecx_readeeprom2(ecx_contextt *context, uint16 slave, int timeout)
  * @param[in] idx         = Used datagram index.
  * @param[in] data        = Pointer to process data segment.
  * @param[in] length      = Length of data segment in bytes.
+ * @param[in] DCO         = Offset position of DC frame.
  */
-static void ecx_pushindex(ecx_contextt *context, uint8 idx, void *data, uint16 length)
+static void ecx_pushindex(ecx_contextt *context, uint8 idx, void *data, uint16 length, uint16 DCO)
 {
    if(context->idxstack->pushed < EC_MAXBUF)
    {
       context->idxstack->idx[context->idxstack->pushed] = idx;
       context->idxstack->data[context->idxstack->pushed] = data;
       context->idxstack->length[context->idxstack->pushed] = length;
+      context->idxstack->dcoffset[context->idxstack->pushed] = DCO;
       context->idxstack->pushed++;
    }
 }
@@ -1661,9 +1691,10 @@ static void ecx_clearindex(ecx_contextt *context)  {
  * In order to recombine the slave response, a stack is used.
  * @param[in]  context        = context struct
  * @param[in]  group          = group number
+ * @param[in]  use_overlap_io = flag if overlapped iomap is used
  * @return >0 if processdata is transmitted.
  */
-int ecx_send_processdata_group(ecx_contextt *context, uint8 group)
+static int ecx_main_send_processdata(ecx_contextt *context, uint8 group, boolean use_overlap_io)
 {
    uint32 LogAdr;
    uint16 w1, w2;
@@ -1673,20 +1704,37 @@ int ecx_send_processdata_group(ecx_contextt *context, uint8 group)
    uint8* data;
    boolean first=FALSE;
    uint16 currentsegment = 0;
+   uint32 iomapinputoffset;
+   uint16 DCO;
 
    wkc = 0;
    if(context->grouplist[group].hasdc)
    {
       first = TRUE;
    }
-   length = context->grouplist[group].Obytes + context->grouplist[group].Ibytes;
+
+   /* For overlapping IO map use the biggest */
+   if(use_overlap_io == TRUE)
+   {
+      /* For overlap IOmap make the frame EQ big to biggest part */
+      length = (context->grouplist[group].Obytes > context->grouplist[group].Ibytes) ?
+         context->grouplist[group].Obytes : context->grouplist[group].Ibytes;
+      /* Save the offset used to compensate where to save inputs when frame returns */
+      iomapinputoffset = context->grouplist[group].Obytes;
+   }
+   else
+   {
+      length = context->grouplist[group].Obytes + context->grouplist[group].Ibytes;
+      iomapinputoffset = 0;
+   }
+   
    LogAdr = context->grouplist[group].logstartaddr;
-   if (length)
+   if(length)
    {
 
       wkc = 1;
       /* LRW blocked by one or more slaves ? */
-      if (context->grouplist[group].blockLRW)
+      if(context->grouplist[group].blockLRW)
       {
          /* if inputs available generate LRD */
          if(context->grouplist[group].Ibytes)
@@ -1710,12 +1758,12 @@ int ecx_send_processdata_group(ecx_contextt *context, uint8 group)
                idx = ecx_getindex(context->port);
                w1 = LO_WORD(LogAdr);
                w2 = HI_WORD(LogAdr);
+               DCO = 0;
                ecx_setupdatagram(context->port, &(context->port->txbuf[idx]), EC_CMD_LRD, idx, w1, w2, sublength, data);
                if(first)
                {
-                  context->DCl = sublength;
                   /* FPRMW in second datagram */
-                  context->DCtO = ecx_adddatagram(context->port, &(context->port->txbuf[idx]), EC_CMD_FRMW, idx, FALSE,
+                  DCO = ecx_adddatagram(context->port, &(context->port->txbuf[idx]), EC_CMD_FRMW, idx, FALSE,
                                            context->slavelist[context->grouplist[group].DCnext].configadr,
                                            ECT_REG_DCSYSTIME, sizeof(int64), context->DCtime);
                   first = FALSE;
@@ -1723,7 +1771,7 @@ int ecx_send_processdata_group(ecx_contextt *context, uint8 group)
                /* send frame */
                ecx_outframe_red(context->port, idx);
                /* push index and data pointer on stack */
-               ecx_pushindex(context, idx, data, sublength);
+               ecx_pushindex(context, idx, data, sublength, DCO);
                length -= sublength;
                LogAdr += sublength;
                data += sublength;
@@ -1748,12 +1796,12 @@ int ecx_send_processdata_group(ecx_contextt *context, uint8 group)
                idx = ecx_getindex(context->port);
                w1 = LO_WORD(LogAdr);
                w2 = HI_WORD(LogAdr);
+               DCO = 0;
                ecx_setupdatagram(context->port, &(context->port->txbuf[idx]), EC_CMD_LWR, idx, w1, w2, sublength, data);
                if(first)
                {
-                  context->DCl = sublength;
                   /* FPRMW in second datagram */
-                  context->DCtO = ecx_adddatagram(context->port, &(context->port->txbuf[idx]), EC_CMD_FRMW, idx, FALSE,
+                  DCO = ecx_adddatagram(context->port, &(context->port->txbuf[idx]), EC_CMD_FRMW, idx, FALSE,
                                            context->slavelist[context->grouplist[group].DCnext].configadr,
                                            ECT_REG_DCSYSTIME, sizeof(int64), context->DCtime);
                   first = FALSE;
@@ -1761,7 +1809,7 @@ int ecx_send_processdata_group(ecx_contextt *context, uint8 group)
                /* send frame */
                ecx_outframe_red(context->port, idx);
                /* push index and data pointer on stack */
-               ecx_pushindex(context, idx, data, sublength);
+               ecx_pushindex(context, idx, data, sublength, DCO);
                length -= sublength;
                LogAdr += sublength;
                data += sublength;
@@ -1778,6 +1826,8 @@ int ecx_send_processdata_group(ecx_contextt *context, uint8 group)
          else
          {
             data = context->grouplist[group].inputs;
+            /* Clear offset, don't compensate for overlapping IOmap if we only got inputs */
+            iomapinputoffset = 0;
          }
          /* segment transfer if needed */
          do
@@ -1787,20 +1837,24 @@ int ecx_send_processdata_group(ecx_contextt *context, uint8 group)
             idx = ecx_getindex(context->port);
             w1 = LO_WORD(LogAdr);
             w2 = HI_WORD(LogAdr);
+            DCO = 0;
             ecx_setupdatagram(context->port, &(context->port->txbuf[idx]), EC_CMD_LRW, idx, w1, w2, sublength, data);
             if(first)
             {
-               context->DCl = sublength;
                /* FPRMW in second datagram */
-               context->DCtO = ecx_adddatagram(context->port, &(context->port->txbuf[idx]), EC_CMD_FRMW, idx, FALSE,
+               DCO = ecx_adddatagram(context->port, &(context->port->txbuf[idx]), EC_CMD_FRMW, idx, FALSE,
                                         context->slavelist[context->grouplist[group].DCnext].configadr,
                                         ECT_REG_DCSYSTIME, sizeof(int64), context->DCtime);
                first = FALSE;
             }
             /* send frame */
             ecx_outframe_red(context->port, idx);
-            /* push index and data pointer on stack */
-            ecx_pushindex(context, idx, data, sublength);
+            /* push index and data pointer on stack.
+             * the iomapinputoffset compensate for where the inputs are stored 
+             * in the IOmap if we use an overlapping IOmap. If a regular IOmap
+             * is used it should always be 0.
+             */
+            ecx_pushindex(context, idx, (data + iomapinputoffset), sublength, DCO);      
             length -= sublength;
             LogAdr += sublength;
             data += sublength;
@@ -1811,6 +1865,40 @@ int ecx_send_processdata_group(ecx_contextt *context, uint8 group)
    return wkc;
 }
 
+/** Transmit processdata to slaves.
+* Uses LRW, or LRD/LWR if LRW is not allowed (blockLRW).
+* Both the input and output processdata are transmitted in the overlapped IOmap.
+* The outputs with the actual data, the inputs replace the output data in the
+* returning frame. The inputs are gathered with the receive processdata function.
+* In contrast to the base LRW function this function is non-blocking.
+* If the processdata does not fit in one datagram, multiple are used.
+* In order to recombine the slave response, a stack is used.
+* @param[in]  context        = context struct
+* @param[in]  group          = group number
+* @return >0 if processdata is transmitted.
+*/
+int ecx_send_overlap_processdata_group(ecx_contextt *context, uint8 group)
+{
+   return ecx_main_send_processdata(context, group, TRUE);
+}
+
+/** Transmit processdata to slaves.
+* Uses LRW, or LRD/LWR if LRW is not allowed (blockLRW).
+* Both the input and output processdata are transmitted.
+* The outputs with the actual data, the inputs have a placeholder.
+* The inputs are gathered with the receive processdata function.
+* In contrast to the base LRW function this function is non-blocking.
+* If the processdata does not fit in one datagram, multiple are used.
+* In order to recombine the slave response, a stack is used.
+* @param[in]  context        = context struct
+* @param[in]  group          = group number
+* @return >0 if processdata is transmitted.
+*/
+int ecx_send_processdata_group(ecx_contextt *context, uint8 group)
+{
+   return ecx_main_send_processdata(context, group, FALSE);
+}
+
 /** Receive processdata from slaves.
  * Second part from ec_send_processdata().
  * Received datagrams are recombined with the processdata with help from the stack.
@@ -1827,51 +1915,51 @@ int ecx_receive_processdata_group(ecx_contextt *context, uint8 group, int timeou
    uint16 le_wkc = 0;
    int valid_wkc = 0;
    int64 le_DCtime;
-   boolean first = FALSE;
+   ec_idxstackT *idxstack;
+   ec_bufT *rxbuf;
 
-   if(context->grouplist[group].hasdc)
-   {
-      first = TRUE;
-   }
+   /* just to prevent compiler warning for unused group */
+   wkc2 = group;
+
+   idxstack = context->idxstack;
+   rxbuf = context->port->rxbuf;
    /* get first index */
    pos = ecx_pullindex(context);
    /* read the same number of frames as send */
    while (pos >= 0)
    {
-      idx = context->idxstack->idx[pos];
-      wkc2 = ecx_waitinframe(context->port, context->idxstack->idx[pos], timeout);
+      idx = idxstack->idx[pos];
+      wkc2 = ecx_waitinframe(context->port, idx, timeout);
       /* check if there is input data in frame */
       if (wkc2 > EC_NOFRAME)
       {
-         if((context->port->rxbuf[idx][EC_CMDOFFSET]==EC_CMD_LRD) || (context->port->rxbuf[idx][EC_CMDOFFSET]==EC_CMD_LRW))
+         if((rxbuf[idx][EC_CMDOFFSET]==EC_CMD_LRD) || (rxbuf[idx][EC_CMDOFFSET]==EC_CMD_LRW))
          {
-            if(first)
+            if(idxstack->dcoffset[pos] > 0)
             {
-               memcpy(context->idxstack->data[pos], &(context->port->rxbuf[idx][EC_HEADERSIZE]), context->DCl);
-               memcpy(&le_wkc, &(context->port->rxbuf[idx][EC_HEADERSIZE + context->DCl]), EC_WKCSIZE);
+               memcpy(idxstack->data[pos], &(rxbuf[idx][EC_HEADERSIZE]), idxstack->length[pos]);
+               memcpy(&le_wkc, &(rxbuf[idx][EC_HEADERSIZE + idxstack->length[pos]]), EC_WKCSIZE);
                wkc = etohs(le_wkc);
-               memcpy(&le_DCtime, &(context->port->rxbuf[idx][context->DCtO]), sizeof(le_DCtime));
+               memcpy(&le_DCtime, &(rxbuf[idx][idxstack->dcoffset[pos]]), sizeof(le_DCtime));
                *(context->DCtime) = etohll(le_DCtime);
-               first = FALSE;
             }
             else
             {
                /* copy input data back to process data buffer */
-               memcpy(context->idxstack->data[pos], &(context->port->rxbuf[idx][EC_HEADERSIZE]), context->idxstack->length[pos]);
+               memcpy(idxstack->data[pos], &(rxbuf[idx][EC_HEADERSIZE]), idxstack->length[pos]);
                wkc += wkc2;
             }
             valid_wkc = 1;
          }
-         else if(context->port->rxbuf[idx][EC_CMDOFFSET]==EC_CMD_LWR)
+         else if(rxbuf[idx][EC_CMDOFFSET]==EC_CMD_LWR)
          {
-            if(first)
+            if(idxstack->dcoffset[pos] > 0)
             {
-               memcpy(&le_wkc, &(context->port->rxbuf[idx][EC_HEADERSIZE + context->DCl]), EC_WKCSIZE);
+               memcpy(&le_wkc, &(rxbuf[idx][EC_HEADERSIZE + idxstack->length[pos]]), EC_WKCSIZE);
                /* output WKC counts 2 times when using LRW, emulate the same for LWR */
                wkc = etohs(le_wkc) * 2;
-               memcpy(&le_DCtime, &(context->port->rxbuf[idx][context->DCtO]), sizeof(le_DCtime));
+               memcpy(&le_DCtime, &(rxbuf[idx][idxstack->dcoffset[pos]]), sizeof(le_DCtime));
                *(context->DCtime) = etohll(le_DCtime);
-               first = FALSE;
             }
             else
             {
@@ -1903,6 +1991,11 @@ int ecx_send_processdata(ecx_contextt *context)
    return ecx_send_processdata_group(context, 0);
 }
 
+int ecx_send_overlap_processdata(ecx_contextt *context)
+{
+   return ecx_send_overlap_processdata_group(context, 0);
+}
+
 int ecx_receive_processdata(ecx_contextt *context, int timeout)
 {
    return ecx_receive_processdata_group(context, 0, timeout);
@@ -2063,7 +2156,7 @@ int ec_writestate(uint16 slave)
  * This is a blocking function.
  * @param[in] slave       = Slave number, 0 = all slaves
  * @param[in] reqstate    = Requested state
- * @param[in] timeout     = Timout value in us
+ * @param[in] timeout     = Timeout value in us
  * @return Requested state, or found state after timeout.
  * @see ecx_statecheck
  */
@@ -2257,6 +2350,23 @@ int ec_send_processdata_group(uint8 group)
    return ecx_send_processdata_group (&ecx_context, group);
 }
 
+/** Transmit processdata to slaves.
+* Uses LRW, or LRD/LWR if LRW is not allowed (blockLRW).
+* Both the input and output processdata are transmitted in the overlapped IOmap.
+* The outputs with the actual data, the inputs replace the output data in the
+* returning frame. The inputs are gathered with the receive processdata function.
+* In contrast to the base LRW function this function is non-blocking.
+* If the processdata does not fit in one datagram, multiple are used.
+* In order to recombine the slave response, a stack is used.
+* @param[in]  group          = group number
+* @return >0 if processdata is transmitted.
+* @see ecx_send_overlap_processdata_group
+*/
+int ec_send_overlap_processdata_group(uint8 group)
+{
+   return ecx_send_overlap_processdata_group(&ecx_context, group);
+}
+
 /** Receive processdata from slaves.
  * Second part from ec_send_processdata().
  * Received datagrams are recombined with the processdata with help from the stack.
@@ -2276,6 +2386,11 @@ int ec_send_processdata(void)
    return ec_send_processdata_group(0);
 }
 
+int ec_send_overlap_processdata(void)
+{
+   return ec_send_overlap_processdata_group(0);
+}
+
 int ec_receive_processdata(int timeout)
 {
    return ec_receive_processdata_group(0, timeout);
